/**
  @Company
    Microchip Technology Inc.

  @Description
    This Source file provides APIs.
    Generation Information :
    Driver Version    :   1.0.0
*/
${disclaimer}

#include "../${header.name}"
#include <stdbool.h>

/* I2C Internal API's */
/* Client */
static inline bool ${componentName}_ClientIsAddressInterrupt(void);
static inline bool ${componentName}_ClientIsDataInterrupt(void);
static inline bool ${componentName}_ClientIsStopInterrupt(void);
static inline void ${componentName}_ClientOpen(void);
static inline void ${componentName}_ClientClose(void);
static inline bool ${componentName}_ClientDIR(void);
static inline char ${componentName}_ClientReset(void);
static inline uint8_t ${componentName}_ClientRead(void);
static inline char ${componentName}_ClientWrite(uint8_t data);
static inline bool ${componentName}_ClientIsNack(void);
static inline void ${componentName}_ClientSendAck(void);
static inline void ${componentName}_ClientSendNack(void);
static inline bool ${componentName}_ClientIsBusCollision(void);
static inline bool ${componentName}_ClientIsBusError(void);
static inline bool ${componentName}_ClientIsTxComplete(void);

// Read Event Interrupt Handlers
void ${componentName}_ReadCallback(void);
void (*${componentName}_ReadInterruptHandler)(void);

// Write Event Interrupt Handlers
void ${componentName}_WriteCallback(void);
void (*${componentName}_WriteInterruptHandler)(void);

// Address Event Interrupt Handlers
void ${componentName}_AddressCallback(void);
void (*${componentName}_AddressInterruptHandler)(void);

// Stop Event Interrupt Handlers
void ${componentName}_StopCallback(void);
void (*${componentName}_StopInterruptHandler)(void);

// Bus Collision Event Interrupt Handlers
void ${componentName}_CollisionCallback(void);
void (*${componentName}_CollisionInterruptHandler)(void);

// Bus Error Event Interrupt Handlers
void ${componentName}_BusErrorCallback(void);
void (*${componentName}_BusErrorInterruptHandler)(void);

uint8_t ${api.Initialize.name}(void)
{
    <#if CTRLA??>
    //${CTRLA.comment}
    ${moduleInstance}.${CTRLA.name} = ${CTRLA.valueAsHex};
    
    </#if>
    <#if DBGCTRL??>
    //Debug Run
    ${moduleInstance}.${DBGCTRL.name} = ${DBGCTRL.valueAsHex};
    
    </#if>
    <#if SADDR??>
    //Client Address
    ${moduleInstance}.${SADDR.name} = ${SADDR.valueAsHex};
    
    </#if>
    <#if SADDRMASK??>
    //${SADDRMASK.comment}
    ${moduleInstance}.${SADDRMASK.name} = ${SADDRMASK.valueAsHex};
    
    </#if>
    <#if SCTRLA??>
    //${SCTRLA.comment}
    ${moduleInstance}.${SCTRLA.name} = ${SCTRLA.valueAsHex};
    
    </#if>
    <#if SCTRLB??>
    //${SCTRLB.comment}
    ${moduleInstance}.${SCTRLB.name} = ${SCTRLB.valueAsHex};
    
    </#if>
    <#if SDATA??>
    //Client Data
    ${moduleInstance}.${SDATA.name} = ${SDATA.valueAsHex};
    
    </#if>
    <#if SSTATUS??>
    //${SSTATUS.comment}
    ${moduleInstance}.${SSTATUS.name} = ${SSTATUS.valueAsHex};
    
    </#if>  
    ${api.SetWriteCallback.name}(NULL);
    ${api.SetReadCallback.name}(NULL);
    ${api.SetAddressCallback.name}(NULL);
    ${api.SetStopCallback.name}(NULL);
    ${api.SetCollisionCallback.name}(NULL);
    ${api.SetBusErrorCallback.name}(NULL);
  
    return 0;
}

void ${api.Open.name}(void)
{
    ${componentName}_ClientOpen();
}

void ${api.Close.name}(void)
{
    ${componentName}_ClientClose();
}

ISR(TWI${instanceNumber}_TWIS_vect)
{
    if (${componentName}_ClientIsBusCollision()) {
        ${componentName}_CollisionCallback();
        return;
    }

    if (${componentName}_ClientIsBusError()) {
        ${componentName}_BusErrorCallback();
        return;
    }

    if (${componentName}_ClientIsAddressInterrupt()) {
        ${componentName}_AddressCallback();
        if (${componentName}_ClientDIR()) {
            // Host wishes to read from Client
            ${componentName}_ReadCallback();
            ${componentName}_ClientSendAck();
        }
        return;
    }
    if (${componentName}_ClientIsDataInterrupt()) {
        if (${componentName}_ClientDIR()) {
            // Host wishes to read from Client
            if (!${componentName}_ClientIsNack()) {
                // Received ACK from Host
                ${componentName}_ReadCallback();
                ${componentName}_ClientSendAck();
            } else {
                // Received NACK from Host
                ${api.GoToUnaddressed.name}();
            }
        } else // Host wishes to write to Client
        {
            ${componentName}_WriteCallback();
        }
        return;
    }

    // Check if STOP was received
    if (${componentName}_ClientIsStopInterrupt()) {
        ${componentName}_StopCallback();
        ${componentName}_ClientIsTxComplete(); // To check the status of the transaction
        return;
    }
}


uint8_t ${api.Read.name}(void)
{
    return ${componentName}_ClientRead();
}

void ${api.Write.name}(uint8_t data)
{
    ${componentName}_ClientWrite(data);
}

void ${api.Enable.name}(void)
{
    ${componentName}_ClientOpen();
}

void ${api.SendAck.name}(void)
{
    ${componentName}_ClientSendAck();
}

void ${api.SendNack.name}(void)
{
    ${componentName}_ClientSendNack();
}

void ${api.GoToUnaddressed.name}(void)
{
    // Reset module
    ${componentName}_ClientReset();
}

// Read Event Interrupt Handlers
void ${componentName}_ReadCallback(void)
{
    if (${componentName}_ReadInterruptHandler) {
        ${componentName}_ReadInterruptHandler();
    }
}

void ${api.SetReadCallback.name}(${moduleNameUpperCase}_callback handler)
{
    ${componentName}_ReadInterruptHandler = handler;
}

// Write Event Interrupt Handlers
void ${componentName}_WriteCallback(void)
{
    if (${componentName}_WriteInterruptHandler) {
        ${componentName}_WriteInterruptHandler();
    }
}

void ${api.SetWriteCallback.name}(${moduleNameUpperCase}_callback handler)
{
    ${componentName}_WriteInterruptHandler = handler;
}

// Address Event Interrupt Handlers
void ${componentName}_AddressCallback(void)
{
    if (${componentName}_AddressInterruptHandler) {
        ${componentName}_AddressInterruptHandler();
    }
}

void ${api.SetAddressCallback.name}(${moduleNameUpperCase}_callback handler)
{
    ${componentName}_AddressInterruptHandler = handler;
}

// Stop Event Interrupt Handlers
void ${componentName}_StopCallback(void)
{
    if (${componentName}_StopInterruptHandler) {
        ${componentName}_StopInterruptHandler();
    }
}

void ${api.SetStopCallback.name}(${moduleNameUpperCase}_callback handler)
{
    ${componentName}_StopInterruptHandler = handler;
}

// Bus Collision Event Interrupt Handlers
void ${componentName}_CollisionCallback(void)
{
    if (${componentName}_CollisionInterruptHandler) {
        ${componentName}_CollisionInterruptHandler();
    }
}

void ${api.SetCollisionCallback.name}(${moduleNameUpperCase}_callback handler)
{
    ${componentName}_CollisionInterruptHandler = handler;
}

// Bus Error Event Interrupt Handlers
void ${componentName}_BusErrorCallback(void)
{
    if (${componentName}_BusErrorInterruptHandler) {
        ${componentName}_BusErrorInterruptHandler();
    }
}

void ${api.SetBusErrorCallback.name}(${moduleNameUpperCase}_callback handler)
{
    ${componentName}_BusErrorInterruptHandler = handler;
}


/* Client Configurations */
static inline void ${componentName}_ClientOpen(void)
{
    ${moduleInstance}.SCTRLA |= TWI_ENABLE_bm;
}

static inline void ${componentName}_ClientClose(void)
{
    ${moduleInstance}.SCTRLA &= ~TWI_ENABLE_bm;
}

static inline bool ${componentName}_ClientIsBusCollision(void)
{
    return ${moduleInstance}.SSTATUS & TWI_COLL_bm;
}

static inline bool ${componentName}_ClientIsBusError(void)
{
    return ${moduleInstance}.SSTATUS & TWI_BUSERR_bm;
}

static inline bool ${componentName}_ClientIsAddressInterrupt(void)
{
    return (${moduleInstance}.SSTATUS & TWI_APIF_bm) && (${moduleInstance}.SSTATUS & TWI_AP_bm);
}

static inline bool ${componentName}_ClientIsDataInterrupt(void)
{
    return ${moduleInstance}.SSTATUS & TWI_DIF_bm;
}

static inline bool ${componentName}_ClientIsStopInterrupt(void)
{
    return (${moduleInstance}.SSTATUS & TWI_APIF_bm) && (!(${moduleInstance}.SSTATUS & TWI_AP_bm));
}

static inline bool ${componentName}_ClientDIR(void)
{
    return ${moduleInstance}.SSTATUS & TWI_DIR_bm;
}

static inline void ${componentName}_ClientSendAck(void)
{
    ${moduleInstance}.SCTRLB = TWI_ACKACT_ACK_gc | TWI_SCMD_RESPONSE_gc;
}

static inline void ${componentName}_ClientSendNack(void)
{
    ${moduleInstance}.SCTRLB = TWI_ACKACT_NACK_gc | TWI_SCMD_COMPTRANS_gc;
}

static inline bool ${componentName}_ClientIsNack(void)
{
    return ${moduleInstance}.SSTATUS & TWI_RXACK_bm;
}

static inline bool ${componentName}_ClientIsTxComplete(void)
{
    ${moduleInstance}.SCTRLB = TWI_SCMD_COMPTRANS_gc;
    return ${moduleInstance}.SCTRLB;
}

static inline uint8_t ${componentName}_ClientRead(void)
{
    return ${moduleInstance}.SDATA;
}

static inline char ${componentName}_ClientWrite(uint8_t data)
{
    ${moduleInstance}.SDATA = data;
    ${moduleInstance}.SCTRLB |= TWI_SCMD_RESPONSE_gc;
    return ${moduleInstance}.SDATA;
}

static inline char ${componentName}_ClientReset(void)
{
    ${moduleInstance}.SSTATUS |= (TWI_DIF_bm | TWI_APIF_bm);
    ${moduleInstance}.SCTRLB = TWI_SCMD_COMPTRANS_gc;
    return ${moduleInstance}.SSTATUS;
}