/**
  @Company
    Microchip Technology Inc.

  @Description
    This Source file provides APIs.
    Generation Information :
    Driver Version    :   1.0.0
*/
${disclaimer}

#ifndef ${moduleNameUpperCase}_CLIENT_H
#define ${moduleNameUpperCase}_CLIENT_H

#include <stdbool.h>
#include <stdint.h>
#include "../system/utils/compiler.h"

typedef void(${moduleNameUpperCase}_callback)(void);

/**
 * \brief Initialize I2C interface
 * If module is configured to disabled state, the clock to the I2C is disabled
 * if this is supported by the device's clock system.
 *
 * \return Initialization status.
 * \retval 0 the init was successful
 * \retval 1 the init was not successful
 */
${simpleApi.Initialize};

/**
 * \brief Open the I2C for communication. Enables the module if disabled.
 *
 * \return Nothing
 */
${simpleApi.Open};

/**
 * \brief Close the I2C for communication. Disables the module if enabled.
 * Disables address recognition.
 *
 * \return Nothing
 */
${simpleApi.Close};

/**
 * \brief The function called by the I2C IRQ handler.
 * Can be called in a polling loop in a polled driver.
 *
 * \return Nothing
 */
void ${componentName}_isr(void);

/**
 * \brief Read one byte from the data register of ${componentName}
 *
 * Function will not block if a character is not available, so should
 * only be called when data is available.
 *
 * \return Data read from the ${componentName} module
 */
${simpleApi.Read};

/**
 * \brief Write one byte to the data register of ${componentName}
 *
 * Function will not block if data cannot be safely accepted, so should
 * only be called when safe, i.e. in the read callback handler.
 *
 * \param[in] data The character to write to the I2C
 *
 * \return Nothing
 */
${simpleApi.Write};

/**
 * \brief Enable address recognition in ${componentName}
 * 1. If supported by the clock system, enables the clock to the module
 * 2. Enables the I2C Client functionality  by setting the enable-bit in the HW's control register
 *
 * \return Nothing
 */
${simpleApi.Enable};

/**
 * \brief Send ACK to received address or data. Should
 * only be called when appropriate, i.e. in the callback handlers.
 *
 * \return Nothing
 */
${simpleApi.SendAck};

/**
 * \brief Send NACK to received address or data. Should
 * only be called when appropriate, i.e. in the callback handlers.
 *
 * \return Nothing
 */
${simpleApi.SendNack};

/**
 * \brief Goto unaddressed state. Used to reset I2C HW that are aware
 * of bus state to an unaddressed state.
 *
 * \return Nothing
 */
${simpleApi.GoToUnaddressed};

/**
 * \brief Callback handler for event where Host wishes to read a byte from Client.
 *
 * \return Nothing
 */
${simpleApi.SetReadCallback};

/**
 * \brief Callback handler for event where Host wishes to write a byte to Client.
 *
 * \return Nothing
 */
${simpleApi.SetWriteCallback};

/**
 * \brief Callback handler for event where Client has received its address.
 *
 * \return Nothing
 */
${simpleApi.SetAddressCallback};

/**
 * \brief Callback handler for event where Client has received a STOP condition after being addressed.
 *
 * \return Nothing
 */
${simpleApi.SetStopCallback};

/**
 * \brief Callback handler for event where Client detects a bus collision.
 *
 * \return Nothing
 */
${simpleApi.SetCollisionCallback};

/**
 * \brief Callback handler for event where Client detects a bus error.
 *
 * \return Nothing
 */
${simpleApi.SetBusErrorCallback};

#endif /* ${moduleNameUpperCase}_CLIENT_H */
