/**
  ${moduleNameUpperCase} Generated Driver API Header File

  @Company
    Microchip Technology Inc.

  @File Name
    ${moduleNameLowerCase}.h

  @Summary
    This is the generated driver types header file for the ${moduleNameUpperCase} driver.

  @Description
    This file provides common enumerations for ${moduleNameUpperCase} driver.
    Generation Information :
        Product Revision  :  ${productName} - ${productVersion}
        Device            :  ${selectedDevice}
        Driver Version    :  1.0.0
    The generated drivers are tested against the following:
        Compiler          :  ${compiler}
        MPLAB             :  ${tool}
*/

${disclaimer}

#ifndef ${moduleNameUpperCase}_H
#define ${moduleNameUpperCase}_H

/**
  Section: Included Files
*/

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include "../system/utils/compiler.h"
#include "${moduleNameLowerCase}_host_types.h"
#include "${moduleNameLowerCase}_host_interface.h"

/**
 Section: Data Type Definitions
*/

/**
  ${moduleGroupNameUpperCase}_HOST_INTERFACE object

  @Description
    Object for ${moduleGroupNameUpperCase}_HOST_INTERFACE.
 */
extern const struct ${moduleGroupNameUpperCase}_HOST_INTERFACE ${componentNameLowerCase}_host_Interface;

#define ${componentNameUpperCase}_BAUD(F_SCL, T_RISE)    \
    ((((((float) ${CLK_PER} / (float)F_SCL)) - 10 - ((float)${CLK_PER} * T_RISE / 1000000))) / 2)

typedef enum {
    I2C${instanceNumber}_NOERR,  // The message was sent.
    I2C${instanceNumber}_BUSY,   // Message was NOT sent, bus was busy.
    I2C${instanceNumber}_FAIL    // Message was NOT sent, bus failure
                // If you are interested in the failure reason,
                // Sit on the event call-backs.
} ${componentNameLowerCase}_error_t;

typedef enum { I2C${instanceNumber}_STOP = 1, I2C${instanceNumber}_RESTART_READ, I2C${instanceNumber}_RESTART_WRITE, I2C${instanceNumber}_CONTINUE, I2C${instanceNumber}_RESET_LINK } ${componentNameLowerCase}_operations_t;

typedef ${componentNameLowerCase}_operations_t (*${componentNameLowerCase}_callback_t)(void *funPtr);

typedef uint16_t ${componentNameLowerCase}_address_t;
//typedef ${componentNameLowerCase}_address_t i2c_address_t;
// common callback responses
${simpleApi.CallbackReturnStop};
${simpleApi.CallbackReturnReset};
${simpleApi.CallbackRestartWrite};
${simpleApi.CallbackRestartRead};


/**
 * \brief Open the I2C for communication
 *
 * \param[in] address The Client address to use in the transfer
 *
 * \return Initialization status.
 * \retval I2C_NOERR The I2C open was successful
 * \retval I2C_BUSY  The I2C open failed because the interface is busy
 * \retval I2C_FAIL  The I2C open failed with an error
 */
${simpleApi.Open};

/**
 * \brief Close the I2C interface
 *
 * \return Status of close operation.
 * \retval I2C_NOERR The I2C open was successful
 * \retval I2C_BUSY  The I2C open failed because the interface is busy
 * \retval I2C_FAIL  The I2C open failed with an error
 */
${simpleApi.Close};

/**
 * \brief Start an Write operation on an opened I2C interface
 *
  *
 * \return Status of operation
 * \retval I2C_NOERR The I2C open was successful
 * \retval I2C_BUSY  The I2C open failed because the interface is busy
 * \retval I2C_FAIL  The I2C open failed with an error
 */
${simpleApi.HostWrite};

/**
 * \brief Start an Read operation on an opened ${componentNameUpperCase} interface
 *
 *
 * \return Status of operation
 * \retval ${componentNameUpperCase}_NOERR The ${componentNameUpperCase} open was successful
 * \retval ${componentNameUpperCase}_BUSY  The ${componentNameUpperCase} open failed because the interface is busy
 * \retval ${componentNameUpperCase}_FAIL  The ${componentNameUpperCase} open failed with an error
 */
${simpleApi.HostRead};

/**
 * \brief Set timeout to be used for I2C operations. Uses the Timeout driver.
 *
 * \param[in] to Timeout in ticks
 *
 * \return Nothing
 */
${simpleApi.SetTimeout};

/**
 * \brief Sets up the data buffer to use, and number of bytes to transfer
 *
 * \param[in] buffer Pointer to data buffer to use for read or write data
 * \param[in] bufferSize Number of bytes to read or write from Client
 *
 * \return Nothing
 */
${simpleApi.SetBuffer};

// Event Callback functions.

/**
 * \brief Set callback to be called when all specifed data has been transferred.
 *
 * \param[in] cb Pointer to callback function
 * \param[in] p  Pointer to the callback function's parameters
 *
 * \return Nothing
 */
${simpleApi.SetDataCompleteCallback};

/**
 * \brief Set callback to be called when there has been a bus collision and arbitration was lost.
 *
 * \param[in] cb Pointer to callback function
 * \param[in] p  Pointer to the callback function's parameters
 *
 * \return Nothing
 */
${simpleApi.SetWriteCollisionCallback};

/**
 * \brief Set callback to be called when the transmitted address was NACK'ed.
 *
 * \param[in] cb Pointer to callback function
 * \param[in] p  Pointer to the callback function's parameters
 *
 * \return Nothing
 */
${simpleApi.SetAddressNackCallback};

/**
 * \brief Set callback to be called when the transmitted data was NACK'ed.
 *
 * \param[in] cb Pointer to callback function
 * \param[in] p  Pointer to the callback function's parameters
 *
 * \return Nothing
 */
${simpleApi.SetDataNackCallback};

/**
 * \brief Set callback to be called when there was a bus timeout.
 *
 * \param[in] cb Pointer to callback function
 * \param[in] p  Pointer to the callback function's parameters
 *
 * \return Nothing
 */
${simpleApi.SetTimeoutCallback};

/**
  @Description
    This routine initializes the I2C1 module.
    This routine must be called before any other I2C1 routine is called.
    This routine should only be called once during system initialization.

  @Preconditions
    None

  @Param
    None

  @Returns
    None

  @Example
    None
*/
void ${componentNameUpperCase}_Initialize(void);

/**
@Description
    This function writes data to a Client on the bus. The function will attempt
    to write length number of bytes from data buffer to a Client whose address
    is specified by address. The I2C Host will generate a Start condition,
    write the data and then generate a Stop Condition. If the Client NACK the request 
    or a bus error was encountered on the bus, the transfer is terminated. The 
    application can call the ${componentNameUpperCase}_ErrorGet() function to know that 
    cause of the error.

    The function is non-blocking. It initiates bus activity and returns
    immediately. The transfer is then completed in the peripheral interrupt. A
    transfer request cannot be placed when another transfer is in progress.
    Calling the write function when another function is already in progress will
    cause the function to return false.

  @Preconditions
    ${componentNameUpperCase}_Initialize() must have been called for the associated

  @Param
    address     - 7-bit / 10-bit Client address.
    data        - pointer to source data buffer that contains the data to be
                  transmitted.
    dataLength  - length of data buffer in number of bytes. Also the number of bytes
                  to be written.

  @Returns
    true  - The request was placed successfully and the bus activity was
            initiated.
    false - The request fails,if there was already a transfer in progress when this function
            was called

  @Example
    None
*/
bool ${componentNameUpperCase}_Write(uint16_t address, uint8_t *data, size_t dataLength);

/**
 * \brief Set Address to be called when there was aa address reception.
 *
 * \param[in] address Loads the address of the host
 *
 * \return Nothing
 */
${simpleApi.SetAddress};

/**
  @Description
    This function writes data to a Client on the bus. The function will attempt
    to write length number of bytes from data buffer to a Client whose address
    is specified by address. The I2C Host will generate a Start condition,
    write the data and then generate a Stop Condition. If the Client NACK the request 
    or a bus error was encountered on the bus, the transfer is terminated. The 
    application can call the ${componentNameUpperCase}_ErrorGet() function to know that 
    cause of the error.

    The function is non-blocking. It initiates bus activity and returns
    immediately. The transfer is then completed in the peripheral interrupt. A
    transfer request cannot be placed when another transfer is in progress.
    Calling the write function when another function is already in progress will
    cause the function to return false.

  @Preconditions
    ${componentNameUpperCase}_Initialize() must have been called for the associated

  @Param
    address     - 7-bit / 10-bit Client address.
    data        - pointer to source data buffer that contains the data to be
                  transmitted.
    dataLength  - length of data buffer in number of bytes. Also the number of bytes
                  to be written.

  @Returns
    true  - The request was placed successfully and the bus activity was
            initiated.
    false - The request fails,if there was already a transfer in progress when this function
            was called

  @Example
    None
*/
bool ${componentNameUpperCase}_Write(uint16_t address, uint8_t *data, size_t dataLength);

/**
  @Description
    This function reads the data from a Client on the bus. The function will
    attempt to read length number of bytes into data buffer from a Client whose
    address is specified as address. The I2C Host generate a Start condition,
    read the data and then generate a Stop Condition.
    If the Client NAKs the request or a bus error is encountered on the bus, the
    transfer is terminated. The application can call ${componentNameUpperCase}_ErrorGet()
    function to know that cause of the error.

    The function is non-blocking. It initiates bus activity and returns
    immediately. The transfer is completed in the peripheral interrupt. A
    transfer request cannot be placed when another transfer is in progress.
    Calling the read function when another function is already in progress will
    cause the function to return false.

  @Preconditions
    ${componentNameUpperCase}_Initialize() must have been called for the associated

  @Param
    address     - 7-bit / 10-bit Client address.
    data        - pointer to source data buffer that contains the data to be
                  transmitted.
    dataLength  - length of data buffer in number of bytes. Also the number of bytes
                  to be written.

  @Returns
    true  - The request was placed successfully and the bus activity was
            initiated.
    false - The request fails,if there was already a transfer in progress when this function
            was called

  @Example
    None
*/
bool ${componentNameUpperCase}_Read(uint16_t address, uint8_t *data, size_t dataLength);

/**
  @Description
    This function writes data from the writeData to the bus and then reads data from
    the Client and stores the received in the readData. The function generates a
    Start condition on the bus and will then send writeLength number of bytes
    contained in writeData. The function will then insert a Repeated start condition
    and proceed to read readLength number of bytes from the client. The received
    bytes are stored in readData buffer. A Stop condition is generated after the
    last byte has been received.

    If the Client NAKs the request or a bus error was encountered on the bus, 
    the transfer is terminated. The application can call ${componentNameUpperCase}_ErrorGet()
    function to know that cause of the error.

    The function is non-blocking. It initiates bus activity and returns
    immediately. The transfer is then completed in the peripheral interrupt. A
    transfer request cannot be placed when another transfer is in progress.
    Calling this function when another function is already in progress will
    cause the function to return false.

  @Preconditions
    ${componentNameUpperCase}_Initialize() must have been called for the associated

  @Param
    address     - 7-bit / 10-bit Client address.
    writeData   - pointer to write data buffer.
    writeLength - write data length in bytes.
    readData    - pointer to read data buffer.
    readLength  - read data length in bytes.

  @Returns
    true  - The request was placed successfully and the bus activity was
            initiated.
    false - The request fails,if there was already a transfer in progress when this function
            was called

  @Example
    None
*/
bool ${componentNameUpperCase}_WriteRead(uint16_t address, uint8_t *writeData, size_t writeLength, uint8_t *readData, size_t readLength);

/**
  @Description
    This function dynamically setup the I2C clock speed (100KHZ to 1MHZ).

  @Preconditions
    ${componentNameUpperCase}_Initialize() must have been called for the associated

  @Param
    setup - Pointer to the structure containing the transfer setup.
    srcClkFreq - I2C Peripheral Clock Source Frequency (FCY = FOSC/2).

  @Returns
    true - Transfer setup was updated Successfully.
    false - Failure while updating transfer setup.

  @Example
    None
*/
bool ${componentNameUpperCase}_TransferSetup(struct ${moduleGroupNameUpperCase}_TRANSFER_SETUP* setup, uint32_t srcClkFreq);

/**
  @Description
    This function the error occurred during I2C Transmit and Receive.

  @Preconditions
    ${componentNameUpperCase}_Initialize() must have been called for the associated

  @Param
    None

  @Returns
    ${moduleGroupNameUpperCase}_ERROR_NONE          - No Error
    ${moduleGroupNameUpperCase}_ERROR_NACK          - Client returned NACK
    ${moduleGroupNameUpperCase}_ERROR_BUS_COLLISION - Bus Collision Error

  @Example
    None
*/
enum ${moduleGroupNameUpperCase}_ERROR ${componentNameUpperCase}_ErrorGet(void);

/**
  @Description
    This function true if the I2C module is busy with a transfer. 
    The application can use the function to check if I2C module is busy before 
    calling any of the data transfer functions. 

    The library does not allow a data transfer operation if another transfer 
    operation is already in progress.

  @Preconditions
    ${componentNameUpperCase}_Initialize() must have been called for the associated

  @Param
    None

  @Returns
    true    - Busy.
    false   - Not Busy.

  @Example
    None
*/
bool ${componentNameUpperCase}_IsBusy(void);

/**
  @Description
    This routine is callback for I2C Bus transfer events, Bus collision and Write 
    collision occur

  @Preconditions
    ${componentNameUpperCase}_Initialize() must have been called for the associated

  @Param
    None

  @Returns
    None   

  @Example
    None
*/
void ${componentNameUpperCase}_Callback(void);

/**
  @Description
    This function sets the pointer to be called "back" when
    the given I2C transfer events, Bus collision and Write collision occur. 

  @Preconditions
    ${componentNameUpperCase}_Initialize() must have been called for the associated

  @Param
    handler - A pointer to a function. 
              Setting this to NULL disables the callback feature.

  @Returns
    None

  @Example
    None
*/
void ${componentNameUpperCase}_CallbackRegister(void (*handler)(void));


/**
  @Description
    This function is common ISR function for all ${componentNameUpperCase} interrupts

  @Preconditions
    ${componentNameUpperCase}_Initialize() must have been called for the associated

  @Param
    None

  @Returns
    None

  @Example
    None
*/
void ${componentNameUpperCase}_ISR(void);


#endif //${moduleNameUpperCase}_H